QuantumSimulator

Struct QuantumSimulator 

pub struct QuantumSimulator { /* private fields */ }
A simulator for a quantum computer

This type represents a quantum computer.

Memory consumption scales 2^n with a given qubits number n. Thus, usually quantum machines up to around 30 qubits can be simulated although internal implementation is usize.

Examples

An example to set the given qubit state to the desired one;

use rusq::prelude::*;

fn set(sim: &mut QuantumSimulator, qubit: &Qubit, r: MeasuredResult) {
    if sim.measure(qubit) != r {
        sim.X(qubit);
    }
}

let mut sim = QuantumSimulator::new(1);
let qubit = &sim.get_qubits()[0];
set(&mut sim, qubit, MeasuredResult::Zero);

assert_eq!(sim.measure(qubit), MeasuredResult::Zero);

As is discussed in new method, the initial values of the qubits are not definite (although in the code one can find some value). Thus, one needs to initialize qubits in this way. This behavior is because not the wavefunction itself but the ray corresponds to the physical state.

Implementations

Source

impl QuantumSimulator

pub fn new(n: usize) -> QuantumSimulator

Creates a new instance with a given number of qubits. Note that the initial states of the qubits are not guaranteed to be a definite value.

Examples
use rusq::prelude::*;

// A simulator with 3 qubits.
let sim = QuantumSimulator::new(3);