pub struct QuantumSimulator { /* private fields */ }Expand description
A simulator for a quantum computer
This type represents a quantum computer.
Memory consumption scales 2^n with a given qubits number n.
Thus, usually quantum machines up to around 30 qubits can be simulated
although internal implementation is usize.
Examples
An example to set the given qubit state to the desired one;
use rusq::prelude::*;
fn set(sim: &mut QuantumSimulator, qubit: &Qubit, r: MeasuredResult) {
if sim.measure(qubit) != r {
sim.X(qubit);
}
}
let mut sim = QuantumSimulator::new(1);
let qubit = &sim.get_qubits()[0];
set(&mut sim, qubit, MeasuredResult::Zero);
assert_eq!(sim.measure(qubit), MeasuredResult::Zero);As is discussed in new method, the initial values of the qubits are not
definite (although in the code one can find some value). Thus, one needs to initialize
qubits in this way. This behavior is because not the wavefunction itself but the ray corresponds
to the physical state.
Implementations
Sourceimpl QuantumSimulator
impl QuantumSimulator
Sourcepub fn new(n: usize) -> QuantumSimulator
pub fn new(n: usize) -> QuantumSimulator
Creates a new instance with a given number of qubits. Note that the initial states of the qubits are not guaranteed to be a definite value.
Examples
use rusq::prelude::*;
// A simulator with 3 qubits.
let sim = QuantumSimulator::new(3);Trait Implementations
Sourceimpl DoubleGateApplicator for QuantumSimulator
impl DoubleGateApplicator for QuantumSimulator
Sourcefn apply_double(
&mut self,
matrix: &Array2<Complex<f64>>,
qubit1: &Qubit,
qubit2: &Qubit,
)
fn apply_double( &mut self, matrix: &Array2<Complex<f64>>, qubit1: &Qubit, qubit2: &Qubit, )
An operation for the given unitary matrix
matrix to qubit1 and qubit2fn CNOT(&mut self, qubit1: &Qubit, qubit2: &Qubit)
fn SWAP(&mut self, qubit1: &Qubit, qubit2: &Qubit)
fn SQSWAP(&mut self, qubit1: &Qubit, qubit2: &Qubit)
fn cphase(&mut self, phi: f64, qubit1: &Qubit, qubit2: &Qubit)
Sourceimpl QuantumMachine for QuantumSimulator
impl QuantumMachine for QuantumSimulator
Sourcefn measure(&mut self, qubit: &Qubit) -> MeasuredResult
fn measure(&mut self, qubit: &Qubit) -> MeasuredResult
Measures the given qubit.
Note that the qubit is expected to be projected to the corresponding state.
Sourcefn get_qubits(&self) -> Vec<Qubit>
fn get_qubits(&self) -> Vec<Qubit>
Returns all the qubits in the machine.
Sourceimpl SingleGateApplicator for QuantumSimulator
impl SingleGateApplicator for QuantumSimulator
Sourcefn apply_single(&mut self, matrix: &Array2<Complex<f64>>, qubit: &Qubit)
fn apply_single(&mut self, matrix: &Array2<Complex<f64>>, qubit: &Qubit)
An operation for the given unitary matrix
matrix to qubitfn H(&mut self, qubit: &Qubit)
fn X(&mut self, qubit: &Qubit)
fn Y(&mut self, qubit: &Qubit)
fn Z(&mut self, qubit: &Qubit)
fn ID(&mut self, qubit: &Qubit)
fn phase(&mut self, phi: f64, qubit: &Qubit)
Sourceimpl TripleGateApplicator for QuantumSimulator
impl TripleGateApplicator for QuantumSimulator
Sourcefn apply_triple(
&mut self,
matrix: &Array2<Complex<f64>>,
qubit1: &Qubit,
qubit2: &Qubit,
qubit3: &Qubit,
)
fn apply_triple( &mut self, matrix: &Array2<Complex<f64>>, qubit1: &Qubit, qubit2: &Qubit, qubit3: &Qubit, )
An operation for the given unitary matrix
matrix to qubit1, qubit2 and qubit3fn CCNOT(&mut self, qubit1: &Qubit, qubit2: &Qubit, qubit3: &Qubit)
fn CSWAP(&mut self, qubit1: &Qubit, qubit2: &Qubit, qubit3: &Qubit)